"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.AzureTranslator = void 0;
var node_fetch_1 = __importDefault(require("node-fetch"));
var lodash_1 = require("lodash");
var html_entities_1 = require("html-entities");
var matchers_1 = require("../matchers");
var LANGUAGE_ENDPOINT = 'https://api.cognitive.microsofttranslator.com/languages?api-version=3.0';
var TRANSLATE_ENDPOINT = 'https://api.cognitive.microsofttranslator.com/translate?api-version=3.0';
var AzureTranslator = /** @class */ (function () {
    function AzureTranslator() {
        this.name = 'Azure';
    }
    AzureTranslator.prototype.initialize = function (config, interpolationMatcher, decodeEscapes) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, apiKey, region, _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _a = config.split(','), apiKey = _a[0], region = _a[1];
                        if (!apiKey)
                            throw new Error("Please provide an API key for Azure.");
                        this.apiKey = apiKey;
                        this.region = region;
                        this.interpolationMatcher = interpolationMatcher;
                        _b = this;
                        return [4 /*yield*/, this.getAvailableLanguages()];
                    case 1:
                        _b.supportedLanguages = _c.sent();
                        this.decodeEscapes = decodeEscapes;
                        return [2 /*return*/];
                }
            });
        });
    };
    AzureTranslator.prototype.getAvailableLanguages = function () {
        return __awaiter(this, void 0, void 0, function () {
            var response, supported, keys, simplified;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, node_fetch_1["default"](LANGUAGE_ENDPOINT)];
                    case 1:
                        response = _a.sent();
                        return [4 /*yield*/, response.json()];
                    case 2:
                        supported = (_a.sent());
                        keys = Object.keys(supported.translation).map(function (k) { return k.toLowerCase(); });
                        simplified = keys
                            .filter(function (k) { return k.includes('-'); })
                            .map(function (l) { return l.split('-')[0]; });
                        return [2 /*return*/, new Set(keys.concat(simplified))];
                }
            });
        });
    };
    AzureTranslator.prototype.supportsLanguage = function (language) {
        return this.supportedLanguages.has(language.toLowerCase());
    };
    AzureTranslator.prototype.translateBatch = function (batch, from, to) {
        return __awaiter(this, void 0, void 0, function () {
            var toTranslate, headers, response, _a, _b, data;
            var _this = this;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        toTranslate = batch.map(function (_a) {
                            var key = _a.key, value = _a.value;
                            var _b = matchers_1.replaceInterpolations(value, _this.interpolationMatcher), clean = _b.clean, replacements = _b.replacements;
                            return { key: key, value: value, clean: clean, replacements: replacements };
                        });
                        headers = {
                            'Ocp-Apim-Subscription-Key': this.apiKey,
                            'Content-Type': 'application/json; charset=UTF-8'
                        };
                        if (this.region) {
                            headers['Ocp-Apim-Subscription-Region'] = this.region;
                        }
                        return [4 /*yield*/, node_fetch_1["default"](TRANSLATE_ENDPOINT + "&from=" + from + "&to=" + to + "&textType=html", {
                                method: 'POST',
                                headers: headers,
                                body: JSON.stringify(toTranslate.map(function (c) { return ({ Text: c.clean }); }))
                            })];
                    case 1:
                        response = _c.sent();
                        if (!!response.ok) return [3 /*break*/, 3];
                        _a = Error.bind;
                        _b = 'Azure Translation failed: ';
                        return [4 /*yield*/, response.text()];
                    case 2: throw new (_a.apply(Error, [void 0, _b + (_c.sent())]))();
                    case 3: return [4 /*yield*/, response.json()];
                    case 4:
                        data = (_c.sent());
                        return [2 /*return*/, data.map(function (res, i) {
                                var translated = matchers_1.reInsertInterpolations(res.translations[0].text, toTranslate[i].replacements);
                                return {
                                    key: toTranslate[i].key,
                                    value: toTranslate[i].value,
                                    translated: _this.decodeEscapes ? html_entities_1.decode(translated) : translated
                                };
                            })];
                }
            });
        });
    };
    AzureTranslator.prototype.translateStrings = function (strings, from, to) {
        return __awaiter(this, void 0, void 0, function () {
            var batches, results;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        batches = lodash_1.chunk(strings, 50);
                        return [4 /*yield*/, Promise.all(batches.map(function (batch) { return _this.translateBatch(batch, from, to); }))];
                    case 1:
                        results = _a.sent();
                        return [2 /*return*/, lodash_1.flatten(results)];
                }
            });
        });
    };
    return AzureTranslator;
}());
exports.AzureTranslator = AzureTranslator;
